@@ -148,6 +148,10 @@ class Modeler(ABC):
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast
+        allow_gaps (bool): Whether or not observations should be included for
+            training and evaluation if there is a period without an observation
+            between the period of observations and the last period of the
+            given time horizon.
     """
 
     def __init__(
@@ -161,6 +165,8 @@ class Modeler(ABC):
         validation_col: str = "_validation",
         period_col: str = "_period",
         max_lead_col: str = "_maximum_lead",
+        spell_col: str = "_spell",
+        allow_gaps: bool = False,
     ) -> None:
         """Characterize data for modelling.
 
@@ -185,6 +191,12 @@ class Modeler(ABC):
                 periods since the earliest period in the data.
             max_lead_col (str): Name of the column representing the number of
                 observable future periods.
+            spell_col (str): Name of the column representing the number of
+                previous spells of consecutive observations of the same individual.
+            allow_gaps (bool): Whether or not observations should be included for
+                training and evaluation if there is a period without an observation
+                between the period of observations and the last period of the
+                given time horizon.
         """
         if (config.get("TIME_IDENTIFIER", "") == "") and data is not None:
             config["TIME_IDENTIFIER"] = data.columns[1]
@@ -207,6 +219,8 @@ class Modeler(ABC):
         self.validation_col = validation_col
         self.period_col = period_col
         self.max_lead_col = max_lead_col
+        self.spell_col = spell_col
+        self.allow_gaps = allow_gaps
         self.reserved_cols = [
             self.duration_col,
             self.event_col,
@@ -215,6 +229,7 @@ class Modeler(ABC):
             self.validation_col,
             self.period_col,
             self.max_lead_col,
+            self.spell_col,
         ]
         if self.config:
             self.reserved_cols.append(self.config["INDIVIDUAL_IDENTIFIER"])
@@ -324,13 +339,15 @@ class SurvivalModeler(Modeler):
             periods since the earliest period in the data.
         max_lead_col (str): Name of the column representing the number of
             observable future periods.
+        spell_col (str): Name of the column representing the number of
+            previous spells of consecutive observations of the same individual.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast.
-        objective (str): The LightGBM model objective appropriate for the
-            outcome type, which is "binary" for binary classification.
-        num_class (int): The num_class LightGBM parameter, which is 1 for
-            binary classification.
+        allow_gaps (bool): Whether or not observations should be included for
+            training and evaluation if there is a period without an observation
+            between the period of observations and the last period of the
+            given time horizon.
     """
 
     def __init__(self, **kwargs):
@@ -378,7 +395,7 @@ class SurvivalModeler(Modeler):
                 self.data[self.period_col]
                 == self.data[self.data[self.test_col]][self.period_col].min()
             )
-        predictions = self.predict(subset=subset, cumulative=True)
+        predictions = self.predict(subset=subset, cumulative=(not self.allow_gaps))
         metrics = []
         lead_lengths = np.arange(self.n_intervals) + 1
         for lead_length in lead_lengths:
@@ -395,12 +412,15 @@ class SurvivalModeler(Modeler):
         metrics = pd.DataFrame(metrics, index=lead_lengths)
         metrics.index.name = "Lead Length"
         metrics["Other Metrics:"] = ""
-        concordance_index_value = concordance_index(
-            self.data[subset][[self.duration_col, self.max_lead_col]].min(axis=1),
-            np.sum(predictions, axis=-1),
-            self.data[subset][self.event_col],
-        )
-        metrics["C-Index"] = np.where(metrics.index == 1, concordance_index_value, "")
+        if not self.allow_gaps:
+            concordance_index_value = concordance_index(
+                self.data[subset][[self.duration_col, self.max_lead_col]].min(axis=1),
+                np.sum(predictions, axis=-1),
+                self.data[subset][self.event_col],
+            )
+            metrics["C-Index"] = np.where(
+                metrics.index == 1, concordance_index_value, ""
+            )
         metrics = metrics.dropna()
         return metrics
 
@@ -410,7 +430,9 @@ class SurvivalModeler(Modeler):
             str(i + 1) + "-period Survival Probability" for i in range(self.n_intervals)
         ]
         return pd.DataFrame(
-            self.predict(subset=self.data[self.predict_col], cumulative=True),
+            self.predict(
+                subset=self.data[self.predict_col], cumulative=(not self.allow_gaps)
+            ),
             columns=columns,
             index=(
                 self.data[self.config["INDIVIDUAL_IDENTIFIER"]][
@@ -438,7 +460,7 @@ class SurvivalModeler(Modeler):
             survived for each combination of lead length and group.
         """
         subset = default_subset_to_all(subset, self.data)
-        predictions = self.predict(subset=subset, cumulative=True)
+        predictions = self.predict(subset=subset, cumulative=(not self.allow_gaps))
         actual_durations = self.data[subset][
             [self.duration_col, self.max_lead_col]
         ].min(axis=1)
@@ -519,6 +541,8 @@ class SurvivalModeler(Modeler):
         self, data: pd.DataFrame, time_horizon: int
     ) -> pd.DataFrame:
         """Return only observations where survival would be observed."""
+        if self.allow_gaps:
+            return data[data[self.max_lead_col] > time_horizon]
         return data[(data[self.duration_col] + data[self.event_col] > time_horizon)]
 
     def label_data(self, time_horizon: int) -> pd.DataFrame:
@@ -527,7 +551,25 @@ class SurvivalModeler(Modeler):
         data[self.duration_col] = data[[self.duration_col, self.max_lead_col]].min(
             axis=1
         )
-        data["label"] = data[self.duration_col] > time_horizon
+        if self.allow_gaps:
+            ids = data[
+                [self.config["INDIVIDUAL_IDENTIFIER"], self.config["TIME_IDENTIFIER"]]
+            ]
+            ids[self.config["TIME_IDENTIFIER"]] = (
+                ids[self.config["TIME_IDENTIFIER"]] - time_horizon - 1
+            )
+            ids["label"] = True
+            data = data.merge(
+                ids,
+                how="left",
+                on=[
+                    self.config["INDIVIDUAL_IDENTIFIER"],
+                    self.config["TIME_IDENTIFIER"],
+                ],
+            )
+            data["label"] = data["label"].fillna(False)
+        else:
+            data["label"] = data[self.duration_col] > time_horizon
         return data
 
 
@@ -553,6 +595,8 @@ class StateModeler(Modeler):
             periods since the earliest period in the data.
         max_lead_col (str): Name of the column representing the number of
             observable future periods.
+        spell_col (str): Name of the column representing the number of
+            previous spells of consecutive observations of the same individual.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast.
@@ -671,6 +715,8 @@ class StateModeler(Modeler):
         self, data: pd.DataFrame, time_horizon: int
     ) -> pd.DataFrame:
         """Return only observations where the future state is observed."""
+        if self.allow_gaps:
+            return data[data[self.max_lead_col] > time_horizon].dropna(subset=["label"])
         return data[(data[self.duration_col] > time_horizon)]
 
     def label_data(self, time_horizon: int) -> pd.Series:
@@ -679,11 +725,24 @@ class StateModeler(Modeler):
         data[self.duration_col] = data[[self.duration_col, self.max_lead_col]].min(
             axis=1
         )
-        data["label"] = data.groupby(self.config["INDIVIDUAL_IDENTIFIER"])[
-            self.state_col
-        ].shift(-time_horizon - 1)
+        ids = data[
+            [
+                self.config["INDIVIDUAL_IDENTIFIER"],
+                self.config["TIME_IDENTIFIER"],
+                self.state_col,
+            ]
+        ]
+        ids[self.config["TIME_IDENTIFIER"]] = (
+            ids[self.config["TIME_IDENTIFIER"]] - time_horizon - 1
+        )
+        ids = ids.rename({self.state_col: "label"}, axis=1)
         if self.objective == "multiclass":
-            data["label"] = data["label"].cat.codes
+            ids["label"] = ids["label"].cat.codes
+        data = data.merge(
+            ids,
+            how="left",
+            on=[self.config["INDIVIDUAL_IDENTIFIER"], self.config["TIME_IDENTIFIER"]],
+        )
         return data
 
 
@@ -709,8 +768,35 @@ class ExitModeler(StateModeler):
         self, data: pd.DataFrame, time_horizon: int
     ) -> pd.DataFrame:
         """Return only observations where exit is observed at the given time horizon."""
-        return data[data[self.event_col] & (data[self.duration_col] == time_horizon)]
+        if self.allow_gaps:
+            ids = data[
+                [self.config["INDIVIDUAL_IDENTIFIER"], self.config["TIME_IDENTIFIER"]]
+            ]
+            ids[self.config["TIME_IDENTIFIER"]] = (
+                ids[self.config["TIME_IDENTIFIER"]] - time_horizon - 1
+            )
+            ids["exit"] = False
+            data = data.merge(
+                ids,
+                how="left",
+                on=[
+                    self.config["INDIVIDUAL_IDENTIFIER"],
+                    self.config["TIME_IDENTIFIER"],
+                ],
+            )
+            data = data[data["exit"].isna() & (data[self.max_lead_col] > time_horizon)]
+            data = data.drop("exit", axis=1)
+        else:
+            data = [data[self.event_col] & (data[self.duration_col] == time_horizon)]
+        return data
 
     def label_data(self, time_horizon: int) -> pd.Series:
         """Return data with the exit circumstance for each observation."""
-        return super().label_data(time_horizon - 1)
+        data = self.data.copy()
+        data[self.duration_col] = data[[self.duration_col, self.max_lead_col]].min(
+            axis=1
+        )
+        data["label"] = data.groupby(
+            [self.config["INDIVIDUAL_IDENTIFIER"], self.spell_col]
+        )[self.exit_col].transform("last")
+        return data
