@@ -341,6 +341,11 @@ class FIFEArgParser(argparse.ArgumentParser):
             type=str,
             help="The name of the feature that identifies time periods in the data.",
         )
+        self.add_argument(
+            "--BY_FEATURE",
+            type=str,
+            help="The name of the feature by which to produce separate Metrics.csv files.",
+        )
         self.add_argument(
             "--CATEGORICAL_SUFFIXES",
             type=str,
